#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

#
# TkEngine Program V1.1
# by Satoshi Imai
# 1.1, 07-Jul-2002 modified by Fuhito Suguri
#

# start animation
proc startAnim {varName alabel lbox} {
    upvar #0 $varName w

    if {$w(statusFlag) == 0} {
        killTimer
        set w(statusLabel) {Start}
        return
    }

    set w(statusLabel) {Stop}
    set idx 0
    $lbox delete 0 end
    set w(loop) 0
    setTimer $lbox $varName
    while {$w(statusFlag)} {
        set idx [expr {$idx + $w(skip) + 1}]
        if {$idx >= $w(num)} {set idx 0}
        $alabel configure -image "pic${idx}"
        update
        after $w(delay)
        incr w(loop)
    }
    return
}

# change animation
proc changeAnim {varName fileName} {
    upvar #0 $varName w

    set save_cur [. cget -cursor]
    . configure -cursor watch
    update
    foreach picName [image names] {
        image delete $picName
    }
    set w(num) 0
    while {![catch "image create photo pic$w(num) \
            -file $fileName \
            -format {gif -index $w(num)}"]} {
        incr w(num)
    }
    . configure -cursor $save_cur
    update
    return
}

# set timer
proc setTimer {lbox varName} {
    upvar #0 $varName w

    set w(loop) 0
    after 1000 "snapCount $lbox $varName"
    return
}

# snap count of frames per second
proc snapCount {lbox varName} {
    upvar #0 $varName w

    if {[$lbox index end] <= 100} {
        $lbox insert end "$w(loop) frames/sec."
        $lbox see end
        setTimer $lbox $varName
    }
    return
}

# kill timer
proc killTimer {} {
    set idList [after info]
    foreach id $idList {
        after cancel $id
    }
    return
}

# quit application
proc quitApp {} {
    killTimer
    update
    exit
}

# ------
#  main
# ------
option add *BorderWidth 1

if {$tk_version < 8.4} {
    tk_messageBox \
        -icon    error \
        -title   "Version Error!" \
        -type    ok \
        -message {Tcl/Tk > 8.4 is required!}
    exit
}
wm title . TkEngine
wm deiconify .
wm protocol . WM_DELETE_WINDOW quitApp

set myself [info script]
# this is for symbolic link on UNIX
while {[file type $myself] eq "link"} {
    set myself [file readlink $myself]
}
set dir [file dirname $myself]
cd $dir
set files [lsort [glob *.gif]]

# initialization
changeAnim info [lindex $files 0]
set info(loop) 0
set info(delay) 50

# gui
set f [frame .f1]
pack $f

set start [checkbutton $f.start \
        -indicatoron  no \
        -selectcolor  honeydew \
        -text         {Start} \
        -padx         5 \
        -pady         5 \
        -textvariable info(statusLabel) \
        -variable     info(statusFlag) \
        -onvalue      1 \
        -offvalue     0 \
        -command      {startAnim info .anim .snap}]
set label1 [label $f.label1 -text {Delay:}]
set delay [scale $f.delay \
        -width    12 \
        -from     0 \
        -to       100 \
        -variable info(delay) \
        -orient   horizontal]
set label2 [label $f.label2 -text {Skip:}]
set skip $f.skip
tk_optionMenu $skip info(skip) 0 1 2 3 4 5 6 7 8 9 10
pack $start $label1 $delay $label2 $skip -side left -padx 5

set anim [label .anim -image pic0 -relief sunken]
pack $anim -side left -fill both -expand 1

set snap [listbox .snap \
        -width              12 \
        -highlightthickness 0 \
        -yscrollcommand     {.sh set}]
set sh [scrollbar .sh -command {.snap yview}]
pack $snap $sh -side left -fill y

# menu
menu .popup -tearoff no
set sel [lindex $files 0]
foreach m $files {
    .popup add radiobutton \
            -label    $m \
            -variable sel \
            -value    $m \
            -command  "changeAnim info $m" 
}

bind . <3> {tk_popup .popup %X %Y}

